@@ -2,15 +2,14 @@ import argparse
 import glob
 import os
 
-import numpy as np
 import torch
 from tqdm import tqdm
 
-from TTS.speaker_encoder.model import SpeakerEncoder
-from TTS.tts.datasets.preprocess import load_meta_data
-from TTS.tts.utils.speakers import save_speaker_mapping
+from TTS.config import BaseDatasetConfig, load_config
+from TTS.speaker_encoder.utils.generic_utils import setup_model
+from TTS.tts.datasets import load_meta_data
+from TTS.tts.utils.speakers import SpeakerManager
 from TTS.utils.audio import AudioProcessor
-from TTS.utils.io import load_config
 
 parser = argparse.ArgumentParser(
     description='Compute embedding vectors for each wav file in a dataset. If "target_dataset" is defined, it generates "speakers.json" necessary for training a multi-speaker model.'
@@ -22,14 +21,14 @@ parser.add_argument(
     help="Path to config file for training.",
 )
 parser.add_argument("data_path", type=str, help="Data path for wav files - directory or CSV file")
-parser.add_argument("output_path", type=str, help="path for training outputs.")
+parser.add_argument("output_path", type=str, help="path for output speakers.json.")
 parser.add_argument(
     "--target_dataset",
     type=str,
     default="",
     help="Target dataset to pick a processor from TTS.tts.dataset.preprocess. Necessary to create a speakers.json file.",
 )
-parser.add_argument("--use_cuda", type=bool, help="flag to set cuda.", default=False)
+parser.add_argument("--use_cuda", type=bool, help="flag to set cuda.", default=True)
 parser.add_argument("--separator", type=str, help="Separator used in file if CSV is passed for data_path", default="|")
 args = parser.parse_args()
 
@@ -44,10 +43,9 @@ sep = args.separator
 if args.target_dataset != "":
     # if target dataset is defined
     dataset_config = [
-        {"name": args.target_dataset, "path": args.data_path, "meta_file_train": None, "meta_file_val": None},
+        BaseDatasetConfig(name=args.target_dataset, path=args.data_path, meta_file_train=None, meta_file_val=None),
     ]
     wav_files, _ = load_meta_data(dataset_config, eval_split=False)
-    output_files = [wav_file[1].replace(data_path, args.output_path).replace(".wav", ".npy") for wav_file in wav_files]
 else:
     # if target dataset is not defined
     if len(split_ext) > 0 and split_ext[1].lower() == ".csv":
@@ -71,13 +69,8 @@ else:
         # Parse all wav files in data_path
         wav_files = glob.glob(data_path + "/**/*.wav", recursive=True)
 
-        output_files = [wav_file.replace(data_path, args.output_path).replace(".wav", ".npy") for wav_file in wav_files]
-
-for output_file in output_files:
-    os.makedirs(os.path.dirname(output_file), exist_ok=True)
-
 # define Encoder model
-model = SpeakerEncoder(**c.model)
+model = setup_model(c)
 model.load_state_dict(torch.load(args.model_path)["model"])
 model.eval()
 if args.use_cuda:
@@ -89,6 +82,8 @@ for idx, wav_file in enumerate(tqdm(wav_files)):
     if isinstance(wav_file, list):
         speaker_name = wav_file[2]
         wav_file = wav_file[1]
+    else:
+        speaker_name = None
 
     mel_spec = ap.melspectrogram(ap.load_wav(wav_file, sr=ap.sample_rate)).T
     mel_spec = torch.FloatTensor(mel_spec[None, :, :])
@@ -96,16 +91,21 @@ for idx, wav_file in enumerate(tqdm(wav_files)):
         mel_spec = mel_spec.cuda()
     embedd = model.compute_embedding(mel_spec)
     embedd = embedd.detach().cpu().numpy()
-    np.save(output_files[idx], embedd)
 
-    if args.target_dataset != "":
-        # create speaker_mapping if target dataset is defined
-        wav_file_name = os.path.basename(wav_file)
-        speaker_mapping[wav_file_name] = {}
-        speaker_mapping[wav_file_name]["name"] = speaker_name
-        speaker_mapping[wav_file_name]["embedding"] = embedd.flatten().tolist()
+    # create speaker_mapping if target dataset is defined
+    wav_file_name = os.path.basename(wav_file)
+    speaker_mapping[wav_file_name] = {}
+    speaker_mapping[wav_file_name]["name"] = speaker_name
+    speaker_mapping[wav_file_name]["embedding"] = embedd.flatten().tolist()
 
-if args.target_dataset != "":
+if speaker_mapping:
     # save speaker_mapping if target dataset is defined
-    mapping_file_path = os.path.join(args.output_path, "speakers.json")
-    save_speaker_mapping(args.output_path, speaker_mapping)
+    if ".json" not in args.output_path:
+        mapping_file_path = os.path.join(args.output_path, "speakers.json")
+    else:
+        mapping_file_path = args.output_path
+    os.makedirs(os.path.dirname(mapping_file_path), exist_ok=True)
+    speaker_manager = SpeakerManager()
+    # pylint: disable=W0212
+    speaker_manager._save_json(mapping_file_path, speaker_mapping)
+    print("Speaker embeddings saved at:", mapping_file_path)
